use ::rltk::prelude::*;

use super::{append_entry, LogFragment};
use crate::colors;

/// The game log builder - builds one line at a time
pub struct Logger {
    current_color: RGB,
    fragments: Vec<LogFragment>,
}

/// Convenience method to create a new [`Logger`] and
/// immediately append text
pub fn line<T: ToString>(text: T) -> Logger {
    Logger::new().append(text)
}

impl Logger {
    pub fn new() -> Self {
        Logger {
            current_color: colors::WHITE,
            fragments: Vec::new(),
        }
    }

    /// Sets the color of text to be added to the current line
    pub fn color(mut self, color: RGB) -> Self {
        self.current_color = color;

        self
    }

    /// The original [`color`](Logger::color) method, using [`rltk`] color constants
    #[allow(dead_code)]
    pub fn rltk_color(mut self, color: (u8, u8, u8)) -> Self {
        self.current_color = RGB::named(color);

        self
    }

    /// Appends text to the current line of the game log.
    /// Generic on the [`ToString`] trait so that [`&str`], [`String`],
    /// and `&String` types, all work without more type juggling
    /// at the call site.
    pub fn append<T: ToString>(mut self, text: T) -> Self {
        self.fragments.push(LogFragment {
            color: self.current_color,
            text: text.to_string(),
        });

        self
    }

    /// Convenience method to append text with a new color
    pub fn append_color<T: ToString>(mut self, color: RGB, text: T) -> Self {
        self.color(color).append(text)
    }

    /// Append the current line to the log output
    pub fn log(self) {
        append_entry(self.fragments);
    }
}
